# SPDX-License-Identifier: LGPL-3.0-or-later
# Copyright (C) 2018 Takashi Sakamoto

from struct import pack, unpack

import gi
gi.require_version('Hinawa', '3.0')
from gi.repository import Hinawa

from hinawa_utils.dice.dice_unit import DiceUnit

__all__ = ['AlesisIoUnit']


class AlesisIoUnit(DiceUnit):
    __OUI_ALESIS = 0x000595

    __SPECS = {
        'iO|14': {
            'analog-in': 4,
            'has_adat_b': False,
        },
        'iO|26': {
            'analog-in': 8,
            'has_adat_b': True,
        },
    }

    __BASE_OFFSET = 0x00200000

    '''
    Map of offset for mixer sources.
              Mixer-1 Mixer-2 Mixer-3 Mixer-4 Mixer-5 Mixer-6 Mixer-7 Mixer-8
    Analog-1:    0038    00b8    0138    01b8    0238    02b8    0338    03b8
    ...
    Analog-8:    0054    00d4    0154    01d4    0254    02d4    0354    03b4
    Stream-1:    0058    00d8    0158    01d8    0258    02d8    0358    03b8
    ...
    Stream-8:    0074    00f4    0174    01f4    0274    02f4    0374    03f4
    ADAT-A-1:    0078    00f8    0178    01f8    0278    02f8    0378    03f8
    ...
    ADAT-A-8:    0094    0114    0194    0214    0294    0314    0394    0414
    ADAT-B-1:    0098    0118    0198    0218    0298    0318    0398    0418
    ...
    ADAT-B-8:    00b4    0134    01b4    0234    02b4    0334    03b4    0434

    Map of offset for mixer outputs.
    Mixer-1:    0438
    ...
    Mixer-8:    043c
    '''

    __MIXER_SRC_MUTE_OFFSETS = {
        'Mixer-1/2': 0x0458,
        'Mixer-3/4': 0x045c,
        'Mixer-5/6': 0x0460,
        'Mixer-7/8': 0x0464,
    }
    __MIXER_OUT_MUTE_OFFSETS = 0x0468
    __MIXER_SRC_LINK_OFFSET = {
        'Mixer-1/2': 0x047c,
        'Mixer-3/4': 0x0450,
        'Mixer-5/6': 0x0454,
        'Mixer-7/8': 0x0458,
    }
    __METER_OFFSET = 0x4c0
    __MIXER_OUT_LEVEL_OFFSET = 0x0564
    __MIXER_23_24_SWITCH = 0x0568
    __SPDIF_OUT_SRC_OFFSET = 0x056c
    __HP34_OUT_SRC_OFFSET = 0x0570
    __MIX_BLEND_OFFSET = 0x0574     # 0x0000 - 0x0100
    __MAIN_LEVEL_OFFSET = 0x0578    # 0x0000 - 0x0100

    __MAX_COEFF = 0x007fffff

    __MIXER_SRC_LABELS = (
        'Analog-1/2',
        'Analog-3/4',
        'Analog-5/6',
        'Analog-7/8',
        'Stream-1/2',
        'Stream-3/4',
        'Stream-5/6',
        'Stream-7/8',
        'ADAT-A-1/2',
        'ADAT-A-3/4',
        'ADAT-A-5/6',
        'ADAT-A-7/8',
        'ADAT-B-1/2',
        'ADAT-B-3/4',
        'ADAT-B-5/6',
        'ADAT-B-7/8',
    )

    __MIXER_LABELS = (
        'Mixer-1/2',
        'Mixer-3/4',
        'Mixer-5/6',
        'Mixer-7/8',
    )

    __LEVEL_LABELS = (
        '-10',
        '+4',
    )

    def __init__(self, fullpath):
        super().__init__(fullpath)

        if self.vendor_id != self.__OUI_ALESIS:
            raise ValueError('Unsupported model.')

        tx_params = self.get_tx_params()
        if tx_params[0]['pcm'] in (4, 6):
            self.name = 'iO|14'
        else:
            self.name = 'iO|26'
        self.__specs = self.__SPECS[self.name]

        # Ensure 23/24 ports of mixer source receive signal from S/PDIF-1/2 if
        # it has no optical interface for ADAT-B.
        if not self.__specs['has_adat_b']:
            data = bytearray(4)
            self.__write_data(self.__MIXER_23_24_SWITCH, data)

    def __write_data(self, offset, data):
        req = Hinawa.FwReq()
        offset += self.__BASE_OFFSET
        self._protocol.write_transactions(req, offset, data)

    def __read_data(self, offset, length):
        req = Hinawa.FwReq()
        offset += self.__BASE_OFFSET
        return self._protocol.read_transactions(req, offset, length)

    def get_mixer_labels(self):
        return self.__MIXER_LABELS

    def get_mixer_src_labels(self):
        labels = []
        for ch in range(1, self.__specs['analog-in'], 2):
            labels.append('Analog-{0}/{1}'.format(ch, ch + 1))
        # MEMO: Any source from stream input receives no influences from
        # out-volume/mute.
        # for ch in range(1, 8, 2):
        #    labels.append('Stream-{0}/{1}'.format(ch, ch + 1))
        if not self.__specs['has_adat_b']:
            for ch in range(1, 8, 2):
                labels.append('ADAT-{0}/{1}'.format(ch, ch + 1))
            labels.append('S/PDIF-1/2')
        else:
            for ch in range(1, 8, 2):
                labels.append('ADAT-A-{0}/{1}'.format(ch, ch + 1))
            for ch in range(1, 6, 2):
                labels.append('ADAT-B-{0}/{1}'.format(ch, ch + 1))
            data = self.__read_data(self.__MIXER_23_24_SWITCH, 4)
            val = unpack('>I', data)[0]
            if val > 0:
                labels.append('ADAT-B-7/8')
            else:
                labels.append('S/PDIF-1/2')
        return labels

    def __calculate_mixer_src_gain_offsets(self, dst, src, src_ch):
        if dst not in self.__MIXER_LABELS:
            raise ValueError('Invalid argument for mixer.')
        if src not in self.get_mixer_src_labels():
            raise ValueError('Invalid argument for input of mixer.')
        if src_ch not in (0, 1):
            raise ValueError('Invalid argument for channel of stereo pair.')

        # iO14 has one optical interface for ADAT.
        if (src.find('ADAT-') == 0 and
                (src.find('ADAT-A-') != 0 or src.find('ADAT-B-') != 0)):
            src = 'ADAT-A-' + src[5:]
        # S/PDIF signal shares destination port with ADAT-B signal.
        if src == 'S/PDIF-1/2':
            src = 'ADAT-B-7/8'

        offset = (self.__MIXER_SRC_LABELS.index(src) * 2 + src_ch) * 4

        offsets = [
            offset + 0x0038 + (self.__MIXER_LABELS.index(dst) * 2) * 0x0080,
            offset + 0x0038 + (self.__MIXER_LABELS.index(dst)
                               * 2 + 1) * 0x0080,
        ]

        return offsets

    def __write_src_pair_values(self, dst, src, src_ch, vals):
        offsets = self.__calculate_mixer_src_gain_offsets(dst, src, src_ch)
        for i, offset in enumerate(offsets):
            data = pack('>I', vals[i])
            self.__write_data(offset, data)

    def __read_src_pair_values(self, dst, src, src_ch):
        vals = [0, 0]
        offsets = self.__calculate_mixer_src_gain_offsets(dst, src, src_ch)
        for i, offset in enumerate(offsets):
            data = self.__read_data(offset, 4)
            vals[i] = unpack('>I', data)[0]
        # normalize.
        total = sum(vals)
        if total > self.__MAX_COEFF:
            vals[0] = vals[0] * self.__MAX_COEFF // total
            vals[1] = vals[1] * self.__MAX_COEFF // total
        return vals

    # -60..0dB
    def __parse_val_to_db(self, val):
        return float(-60.0 + val * 60 / self.__MAX_COEFF)

    def __build_val_from_db(self, db):
        if db > 0:
            raise ValueError('Invalid argument for dB value.')
        return int((db + 60.0) * self.__MAX_COEFF / 60)

    def set_mixer_src_gain(self, dst, src, src_ch, db):
        if src_ch not in (0, 1):
            raise ValueError('Invalid argument for channel of stereo pair.')
        vals = self.__read_src_pair_values(dst, src, src_ch)
        total = sum(vals)
        val = self.__build_val_from_db(db)
        if total == 0:
            vals[src_ch] = val
            vals[(src_ch + 1) % 2] = 0
        else:
            vals[0] = vals[0] * val // total
            vals[1] = vals[1] * val // total
        self.__write_src_pair_values(dst, src, src_ch, vals)

    def get_mixer_src_gain(self, dst, src, src_ch):
        if src_ch not in (0, 1):
            raise ValueError('Invalid argument for channel of stereo pair.')
        vals = self.__read_src_pair_values(dst, src, src_ch)
        db = self.__parse_val_to_db(sum(vals))
        return db

    def set_mixer_src_balance(self, dst, src, src_ch, balance):
        vals = self.__read_src_pair_values(dst, src, src_ch)
        total = sum(vals)
        vals[0] = int(total * (100 - balance) // 100)
        vals[1] = total - vals[0]
        self.__write_src_pair_values(dst, src, src_ch, vals)

    def get_mixer_src_balance(self, dst, src, src_ch):
        vals = self.__read_src_pair_values(dst, src, src_ch)
        total = sum(vals)
        if total == 0:
            balance = src_ch * 100.0
        else:
            balance = float(100 * vals[1] // total)
        return balance

    def set_mixer_src_link(self, dst, src, link):
        if dst not in self.__MIXER_SRC_LINK_OFFSET:
            raise ValueError('Invalid argument for mixer.')
        srcs = self.get_mixer_src_labels()
        if src not in srcs:
            raise ValueError('Invalid argument for source of mixer.')
        data = self.__read_data(self.__MIXER_SRC_LINK_OFFSET[dst], 4)
        val = unpack('>I', data)[0]
        pos = srcs.index(src)
        if link:
            val |= 1 << pos
        else:
            val &= ~(1 << pos)
        data = pack('>I', val)
        self.__write_data(self.__MIXER_SRC_LINK_OFFSET[dst], data)

    def get_mixer_src_link(self, dst, src):
        if dst not in self.__MIXER_SRC_LINK_OFFSET:
            raise ValueError('Invalid argument for mixer.')
        srcs = self.get_mixer_src_labels()
        if src not in srcs:
            raise ValueError('Invalid argument for source of mixer.')
        data = self.__read_data(self.__MIXER_SRC_LINK_OFFSET[dst], 4)
        val = unpack('>I', data)[0]
        pos = srcs.index(src)
        return bool(val & (1 << pos))

    def set_mixer_src_mute(self, dst, src, src_ch, mute):
        if dst not in self.__MIXER_LABELS:
            raise ValueError('Invalid argument for mixer.')
        labels = self.get_mixer_src_labels()
        if src not in labels:
            raise ValueError('Invalid argument for input of mixer.')
        if src_ch not in (0, 1):
            raise ValueError('Invalid argument for channel of stereo pair.')
        offset = self.__MIXER_SRC_MUTE_OFFSETS[dst]
        data = self.__read_data(offset, 4)
        val = unpack('>I', data)[0]
        index = labels.index(src) + src_ch
        if mute:
            val |= (1 << index)
        else:
            val &= ~(1 << index)
        data = pack('>I', val)
        self.__write_data(offset, data)

    def get_mixer_src_mute(self, dst, src, src_ch):
        if dst not in self.__MIXER_LABELS:
            raise ValueError('Invalid argument for mixer.')
        labels = self.get_mixer_src_labels()
        if src not in labels:
            raise ValueError('Invalid argument for input of mixer.')
        if src_ch not in (0, 1):
            raise ValueError('Invalid argument for channel of stereo pair.')
        offset = self.__MIXER_SRC_MUTE_OFFSETS[dst]
        data = self.__read_data(offset, 4)
        val = unpack('>I', data)[0]
        index = labels.index(src) + src_ch
        return bool(val & (1 << index))

    def set_mixer_spdif_src(self, enable):
        if enable:
            val = 0
        else:
            val = 1
        data = pack('>I', val)
        self.__write_data(self.__MIXER_23_24_SWITCH, data)

    def get_mixer_spdif_src(self):
        data = self.__read_data(self.__MIXER_23_24_SWITCH, 4)
        val = unpack('>I', data)[0]
        return bool(val == 0)

    def get_level_labels(self):
        return self.__LEVEL_LABELS

    def __calculate_mixer_out_offset(self, target, ch):
        if target not in self.__MIXER_LABELS:
            raise ValueError('Invalid argument for mixer out')
        if ch not in (0, 1):
            raise ValueError('Invalid argument for channel in stereo pair')
        offset = 0x0438 + (self.__MIXER_LABELS.index(target) * 2 + ch) * 4
        return offset

    def set_mixer_out_volume(self, target, ch, db):
        offset = self.__calculate_mixer_out_offset(target, ch)
        val = self.__build_val_from_db(db)
        data = pack('>I', val)
        self.__write_data(offset, data)

    def get_mixer_out_volume(self, target, ch):
        offset = self.__calculate_mixer_out_offset(target, ch)
        data = self.__read_data(offset, 4)
        val = unpack('>I', data)[0]
        return self.__parse_val_to_db(val)

    def set_mixer_out_level(self, target, ch, level):
        if target not in self.__MIXER_LABELS:
            raise ValueError('Invalid argument for mixer out')
        if ch not in (0, 1):
            raise ValueError('Invalid argument for channel in pair')
        if level not in self.__LEVEL_LABELS:
            raise ValueError('Invalid argument for output level')
        data = self.__read_data(self.__MIXER_OUT_LEVEL_OFFSET, 4)
        val = unpack('>I', data)[0]

        pos = self.__MIXER_LABELS.index(target) * 2 + ch
        val &= ~(1 << pos)
        val |= self.__LEVEL_LABELS.index(level) << pos

        data = pack('>I', val)
        self.__write_data(self.__MIXER_OUT_LEVEL_OFFSET, data)

    def get_mixer_out_level(self, target, ch):
        if target not in self.__MIXER_LABELS:
            raise ValueError('Invalid argument for mixer out')
        if ch not in (0, 1):
            raise ValueError('Invalid argument for channel in pair')
        data = self.__read_data(self.__MIXER_OUT_LEVEL_OFFSET, 4)
        val = unpack('>I', data)[0]

        pos = self.__MIXER_LABELS.index(target) * 2 + ch
        if val & (1 << pos):
            index = 1
        else:
            index = 0

        return self.__LEVEL_LABELS[index]

    def set_mixer_out_mute(self, target, ch, mute):
        if target not in self.__MIXER_LABELS:
            raise ValueError('Invalid argument for mixer out')
        if ch not in (0, 1):
            raise ValueError('Invalid argument for channel in pair')
        data = self.__read_data(self.__MIXER_OUT_MUTE_OFFSETS, 4)
        val = unpack('>I', data)[0]

        pos = self.__MIXER_LABELS.index(target) * 2 + ch
        if mute:
            val |= 1 << pos
        else:
            val &= ~(1 << pos)
        data = pack('>I', val)
        self.__write_data(self.__MIXER_OUT_MUTE_OFFSETS, data)

    def get_mixer_out_mute(self, target, ch):
        if target not in self.__MIXER_LABELS:
            raise ValueError('Invalid argument for mixer out')
        if ch not in (0, 1):
            raise ValueError('Invalid argument for channel in pair')
        data = self.__read_data(self.__MIXER_OUT_MUTE_OFFSETS, 4)
        val = unpack('>I', data)[0]

        pos = self.__MIXER_LABELS.index(target) * 2 + ch
        return bool(val & (1 << pos))

    def get_output_labels(self):
        labels = []
        for ch in range(1, self.__specs['analog-in'], 2):
            labels.append('Analog-{0}/{1}'.format(ch, ch + 1))
        labels.append('Headphone-1/2')
        labels.append('Headphone-3/4')
        labels.append('S/PDIF-1/2')
        return labels

    def get_output_src_labels(self, target):
        labels = []
        if target.find('Analog-') == 0:
            labels.append('Mixer-' + target[7:])
        elif target == 'Headphone-1/2':
            labels.append('Mixer-1/2')
        else:
            labels.extend(self.__MIXER_LABELS)
        return labels

    def set_output_src(self, target, src):
        if target not in self.get_output_labels():
            raise ValueError('Invalid argument for output.')
        srcs = self.get_output_src_labels(target)
        if src not in srcs:
            raise ValueError('Invalid argument for source of output.')
        if len(srcs) == 1:
            return
        if target == 'Headphone-3/4':
            offset = self.__HP34_OUT_SRC_OFFSET
        elif target == 'S/PDIF-1/2':
            offset = self.__SPDIF_OUT_SRC_OFFSET
        else:
            return
        val = srcs.index(src)
        data = pack('>I', val)
        self.__write_data(offset, data)

    def get_output_src(self, target):
        if target not in self.get_output_labels():
            raise ValueError('Invalid argument for output.')
        srcs = self.get_output_src_labels(target)
        if len(srcs) == 1:
            return srcs[0]
        if target == 'Headphone-3/4':
            offset = self.__HP34_OUT_SRC_OFFSET
        elif target == 'S/PDIF-1/2':
            offset = self.__SPDIF_OUT_SRC_OFFSET
        else:
            return None
        data = self.__read_data(offset, 4)
        val = unpack('>I', data)[0]
        return srcs[val]

    def get_meter_labels(self):
        labels = []
        for ch in range(1, 9):
            labels.append('Analog-{0}'.format(ch))
        for ch in range(1, 9):
            labels.append('Stream-{0}'.format(ch))
        if not self.__specs['has_adat_b']:
            for ch in range(1, 9):
                labels.append('ADAT-{0}'.format(ch))
            for ch in range(1, 3):
                labels.append('S/PDIF-{0}'.format(ch))
        else:
            for ch in range(1, 9):
                labels.append('ADAT-A-{0}'.format(ch))
            for ch in range(1, 7):
                labels.append('ADAT-B-{0}'.format(ch))
            data = self.__read_data(self.__MIXER_23_24_SWITCH, 4)
            val = unpack('>I', data)[0]
            if val:
                labels.append('ADAT-B-7/8')
            else:
                labels.append('S/PDIF-1/2')
        for ch in range(1, 9):
            labels.append('Mixer-{0}'.format(ch))
        return labels

    def get_meters(self):
        meters = []
        data = self.__read_data(self.__METER_OFFSET, 160)
        vals = unpack('>40I', data)
        meters.extend(vals[0:24])
        if self.__specs['has_adat_b']:
            meters.extend(vals[25:30])
        meters.extend(vals[30:])
        for i, meter in enumerate(meters):
            meters[i] = self.__parse_val_to_db(meter)
        return meters

    def get_mix_blend_ratio(self):
        data = self.__read_data(self.__MIX_BLEND_OFFSET, 4)
        val = unpack('>I', data)[0]
        return val / 0x100

    def get_main_level_ratio(self):
        data = self.__read_data(self.__MAIN_LEVEL_OFFSET, 4)
        val = unpack('>I', data)[0]
        return val / 0x100



# SPDX-License-Identifier: LGPL-3.0-or-later
# Copyright (C) 2019 Takashi Sakamoto

from hinawa_utils.oxfw.oxfw_unit import OxfwUnit

from hinawa_utils.oxfw.apogee_protocol import (
    MicCmd, InputCmd, OutputCmd, MixerCmd, DisplayCmd, KnobCmd,
)

__all__ = ['ApogeeDuetUnit']


class ApogeeDuetUnit(OxfwUnit):
    def __init__(self, path):
        super().__init__(path)

        if (self.vendor_name != 'Apogee Electronics' or
                self.model_name != 'Duet'):
            raise ValueError('Unsupported model: {0}, {1}'.format(
                self.vendor_name, self.model_name))

    #
    # Mic configurations.
    #
    def get_mic_labels(self):
        return MicCmd.get_mic_labels()

    def set_mic_polarity(self, target: str, enable: bool):
        MicCmd.set_polarity(self.fcp, target, enable)

    def get_mic_polarity(self, target: str):
        return MicCmd.get_polarity(self.fcp, target)

    def set_mic_power(self, target: str, enable: bool):
        MicCmd.set_power(self.fcp, target, enable)

    def get_mic_power(self, target: str):
        return MicCmd.get_power(self.fcp, target)

    #
    # Input configurations.
    #
    def get_in_labels(self):
        return InputCmd.get_target_labels()

    def get_in_level_labels(self):
        return InputCmd.get_level_labels()

    def set_in_level(self, target, level):
        InputCmd.set_level(self.fcp, target, level)

    def get_in_level(self, target):
        return InputCmd.get_level(self.fcp, target)

    def get_in_attr_labels(self):
        return InputCmd.get_attr_labels()

    def set_in_attr(self, target, attr):
        InputCmd.set_attr(self.fcp, target, attr)

    def get_in_attr(self, target):
        return InputCmd.get_attr(self.fcp, target)

    def get_in_src_labels(self):
        return InputCmd.get_src_labels()

    def set_in_src(self, target, src):
        InputCmd.set_src(self.fcp, target, src)

    def get_in_src(self, target):
        return InputCmd.get_src(self.fcp, target)

    def set_in_gain(self, target: str, db: float):
        InputCmd.set_gain(self.fcp, target, db)

    def get_in_gain(self, target: str):
        return InputCmd.get_gain(self.fcp, target)

    def set_in_clickless(self, enable):
        InputCmd.set_clickless(self.fcp, enable)

    def get_in_clickless(self):
        return InputCmd.get_clickless(self.fcp)

    def get_input_meters(self):
        return InputCmd.get_meters(self)

    #
    # Output configurations.
    #
    def get_out_attr_labels(self):
        return OutputCmd.get_attr_labels()

    def set_out_attr(self, attr: str):
        OutputCmd.set_attr(self.fcp, attr)

    def get_out_attr(self):
        return OutputCmd.get_attr(self.fcp)

    def set_out_mute(self, enable: bool):
        OutputCmd.set_mute(self.fcp, enable)

    def get_out_mute(self):
        return OutputCmd.get_mute(self.fcp)

    def set_out_volume(self, db: float):
        OutputCmd.set_volume(self.fcp, db)

    def get_out_volume(self):
        return OutputCmd.get_volume(self.fcp)

    #
    # Route configurations.
    #
    def get_out_src_labels(self):
        return OutputCmd.get_out_src_labels()

    def set_out_src(self, src: str):
        OutputCmd.set_out_src(self.fcp, src)

    def get_out_src(self):
        return OutputCmd.get_out_src(self.fcp)

    #
    # Mixer configurations.
    #
    def get_mixer_src_labels(self):
        return MixerCmd.get_mixer_src_labels()

    def set_mixer_src(self, src, ch, db):
        MixerCmd.set_src_gain(self.fcp, src, ch, db)

    def get_mixer_src(self, src, ch):
        return MixerCmd.get_src_gain(self.fcp, src, ch)

    def get_mixer_meters(self):
        return MixerCmd.get_meters(self)

    #
    # Hardware configurations.
    #
    def get_display_target_labels(self):
        return DisplayCmd.get_target_labels()

    def set_display_target(self, target):
        DisplayCmd.set_target(self.fcp, target)

    def get_display_target(self):
        return DisplayCmd.get_target(self.fcp)

    def get_display_overhold_labels(self):
        return DisplayCmd.get_overhold_labels()

    def set_display_overhold(self, target):
        DisplayCmd.set_overhold(self.fcp, target)

    def get_display_overhold(self):
        return DisplayCmd.get_overhold(self.fcp)

    def set_display_follow(self, enable):
        DisplayCmd.set_follow(self.fcp, enable)

    def get_display_follow(self):
        return DisplayCmd.get_follow(self.fcp)

    def clear_display(self):
        DisplayCmd.reset_meters(self.fcp)

    #
    # Knob states.
    #
    def get_knob_states(self):
        return KnobCmd.get_states(self.fcp)



# SPDX-License-Identifier: LGPL-3.0-or-later
# Copyright (C) 2019 Takashi Sakamoto

from enum import Enum

import gi
gi.require_version('Hinawa', '3.0')
from gi.repository import Hinawa

from hinawa_utils.ta1394.general import AvcGeneral


class VendorCmd(Enum):
    DISPLAY_MODE = 0x10
    CONTROL_MODE = 0x11
    INPUT_MODE = 0x12
    FIRMWARE_VER = 0x13


class TascamProtocol():
    __OUI = (0x00, 0x02, 0x2e)      # TEAC Corporation.

    __PREFIX = (0x46, 0x49, 0x31)   # FI1

    __PARAMS = {
        VendorCmd.DISPLAY_MODE: {
            'always-off':           0x00,
            'always-on':            0x01,
            'breathe':              0x02,
            'metronome':            0x03,
            'MIDI-clock-rotate':    0x04,
            'MIDI-clock-flash':     0x05,
            'jog-slow-rotate':      0x06,
            'jog-track':            0x07,
        },
        VendorCmd.CONTROL_MODE: {
            'native':               0x00,
            'mackie-HUI-emulation': 0x01,
        },
        VendorCmd.INPUT_MODE: {
            'stereo':   0x00,
            'monaural': 0x01,
        },
    }

    @classmethod
    def __command_set(cls, fcp, cmd, key, name):
        if key not in cls.__PARAMS[cmd]:
            raise ValueError('Invalid argument for {0}.'.format(name))
        data = bytearray()
        data.extend(cls.__PREFIX)
        data.append(cmd.value)
        data.append(cls.__PARAMS[cmd][key])
        try:
            AvcGeneral.set_vendor_dependent(fcp, cls.__OUI, data)
            # The response frame includes wrong __OUI and __PREFIX.
        except Exception as e:
            # The response code is 0x0c, against control requect (0x00).
            if str(e) != 'Unknown status':
                raise e

    @classmethod
    def __command_get(cls, fcp, cmd, name):
        data = bytearray()
        data.extend(cls.__PREFIX)
        data.append(cmd.value)
        data.append(0xff)
        resp = AvcGeneral.get_vendor_dependent(fcp, cls.__OUI, data)
        # The response frame includes wrong __OUI and __PREFIX.
        for key, val in cls.__PARAMS[cmd].items():
            if val == resp[4]:
                return key
        raise OSError('Invalid argument for {0}.'.format(name))

    @classmethod
    def get_display_mode_labels(cls):
        return list(cls.__PARAMS[VendorCmd.DISPLAY_MODE].keys())

    @classmethod
    def set_display_mode(cls, fcp: Hinawa.FwFcp, mode: str):
        cls.__command_set(fcp, VendorCmd.DISPLAY_MODE, mode,
                          'mode of display')

    @classmethod
    def get_display_mode(cls, fcp: Hinawa.FwFcp):
        return cls.__command_get(fcp, VendorCmd.DISPLAY_MODE,
                                 'mode of display')

    @classmethod
    def get_control_mode_labels(cls):
        return list(cls.__PARAMS[VendorCmd.CONTROL_MODE].keys())

    @classmethod
    def set_control_mode(cls, fcp: Hinawa.FwFcp, mode: str):
        cls.__command_set(fcp, VendorCmd.CONTROL_MODE, mode,
                          'mode of control')

    @classmethod
    def get_control_mode(cls, fcp: Hinawa.FwFcp):
        return cls.__command_get(fcp, VendorCmd.CONTROL_MODE,
                                 'mode of control')

    @classmethod
    def get_input_mode_labels(cls):
        return list(cls.__PARAMS[VendorCmd.INPUT_MODE].keys())

    @classmethod
    def set_input_mode(cls, fcp: Hinawa.FwFcp, mode: str):
        cls.__command_set(fcp, VendorCmd.INPUT_MODE, mode,
                          'mode of input')

    @classmethod
    def get_input_mode(cls, fcp: Hinawa.FwFcp):
        return cls.__command_get(fcp, VendorCmd.INPUT_MODE,
                                 'mode of input')

    @classmethod
    def get_firmware_version(cls, fcp: Hinawa.FwFcp):
        data = bytearray()
        data.extend(cls.__PREFIX)
        data.append(VendorCmd.FIRMWARE_VER.value)
        data.append(0xff)
        resp = AvcGeneral.get_vendor_dependent(fcp, cls.__OUI, data)
        # The response frame includes wrong __OUI and __PREFIX.
        return resp[4]



# SPDX-License-Identifier: LGPL-3.0-or-later
# Copyright (C) 2018 Takashi Sakamoto

from threading import Thread

import gi
gi.require_version('GLib', '2.0')
gi.require_version('Hinawa', '3.0')
from gi.repository import GLib, Hinawa

from hinawa_utils.dice.tcat_protocol_general import TcatProtocolGeneral
from hinawa_utils.ta1394.config_rom_parser import Ta1394ConfigRomParser

__all__ = ['DiceUnit']


class DiceUnit(Hinawa.SndDice):
    def __init__(self, path):
        super().__init__()
        self.open(path)
        if self.get_property('type') != 1:
            raise ValueError('The character device is not for Dice unit')

        ctx = GLib.MainContext.new()
        self.create_source().attach(ctx)
        self.__unit_dispatcher = GLib.MainLoop.new(ctx, False)
        self.__unit_th = Thread(target=lambda d: d.run(), args=(self.__unit_dispatcher, ))
        self.__unit_th.start()

        node = self.get_node()
        ctx = GLib.MainContext.new()
        node.create_source().attach(ctx)
        self.__node_dispatcher = GLib.MainLoop.new(ctx, False)
        self.__node_th = Thread(target=lambda d: d.run(), args=(self.__node_dispatcher, ))
        self.__node_th.start()

        parser = Ta1394ConfigRomParser()
        info = parser.parse_rom(self.get_node().get_config_rom())
        self.vendor_id = info['vendor-id']
        self.model_id = info['model-id']

        req = Hinawa.FwReq()
        self._protocol = TcatProtocolGeneral(self, req)

    def release(self):
        self.__unit_dispatcher.quit()
        self.__node_dispatcher.quit()
        self.__unit_th.join()
        self.__node_th.join()

    def __enter__(self):
        return self

    def __exit__(self, ex_type, ex_value, trace):
        self.release()

    def get_owner_addr(self):
        req = Hinawa.FwReq()
        return self._protocol.read_owner_addr(req)

    def get_latest_notification(self):
        req = Hinawa.FwReq()
        return self._protocol.read_latest_notification(req)

    def set_nickname(self, name):
        req = Hinawa.FwReq()
        self._protocol.write_nickname(req, name)

    def get_nickname(self):
        req = Hinawa.FwReq()
        return self._protocol.read_nickname(req)

    def get_supported_clock_sources(self):
        labels = []
        clk_srcs = self._protocol.get_supported_clock_sources()
        for i, label in enumerate(self._protocol.get_clock_source_names()):
            if label != 'Unused' and self._protocol.CLOCK_BITS[i] in clk_srcs:
                labels.append(label)
        return labels

    def set_clock_source(self, source):
        if self.get_property('streaming'):
            raise RuntimeError('Packet streaming started.')
        req = Hinawa.FwReq()
        labels = self._protocol.get_clock_source_names()
        if source not in labels or source == 'Unused':
            raise ValueError('Invalid argument for clock source.')
        alias = self._protocol.CLOCK_BITS[labels.index(source)]
        self._protocol.write_clock_source(req, alias)

    def get_clock_source(self):
        req = Hinawa.FwReq()
        labels = self._protocol.get_clock_source_names()
        src = self._protocol.read_clock_source(req)
        index = {v: k for k, v in self._protocol.CLOCK_BITS.items()}[src]
        return labels[index]

    def get_supported_sampling_rates(self):
        return self._protocol.get_supported_sampling_rates()

    def set_sampling_rate(self, rate):
        if self.get_property('streaming'):
            raise RuntimeError('Packet streaming started.')
        req = Hinawa.FwReq()
        self._protocol.write_sampling_rate(req, rate)

    def get_sampling_rate(self):
        req = Hinawa.FwReq()
        return self._protocol.read_sampling_rate(req)

    def get_enabled(self):
        req = Hinawa.FwReq()
        return self._protocol.read_enabled(req)

    def get_clock_status(self):
        req = Hinawa.FwReq()
        return self._protocol.read_clock_status(req)

    def get_external_clock_states(self):
        req = Hinawa.FwReq()
        return self._protocol.read_external_clock_states(req)

    def get_measured_sampling_rate(self):
        req = Hinawa.FwReq()
        return self._protocol.read_measured_sampling_rate(req)

    def get_dice_version(self):
        return self._protocol.get_dice_version()

    def get_tx_params(self):
        req = Hinawa.FwReq()
        return self._protocol.read_tx_params(req)

    def get_rx_params(self):
        req = Hinawa.FwReq()
        return self._protocol.read_rx_params(req)

    def get_external_sync_clock_source(self):
        req = Hinawa.FwReq()
        return self._protocol.read_external_sync_clock_source(req)

    def get_external_sync_locked(self):
        req = Hinawa.FwReq()
        return self._protocol.read_external_sync_locked(req)

    def get_external_sync_rate(self):
        req = Hinawa.FwReq()
        return self._protocol.read_external_sync_rate(req)

    def get_external_sync_adat_status(self):
        req = Hinawa.FwReq()
        return self._protocol.read_external_sync_adat_status(req)



# SPDX-License-Identifier: LGPL-3.0-or-later
# Copyright (C) 2018 Takashi Sakamoto

from threading import Thread

import gi
gi.require_version('GLib', '2.0')
gi.require_version('Hinawa', '3.0')
from gi.repository import GLib, Hinawa

from hinawa_utils.motu.motu_protocol_v1 import MotuProtocolV1
from hinawa_utils.motu.motu_protocol_v2 import MotuProtocolV2
from hinawa_utils.motu.motu_protocol_v3 import MotuProtocolV3
from hinawa_utils.motu.config_rom_parser import MotuConfigRomParser

__all__ = ['MotuUnit']


class MotuUnit(Hinawa.SndMotu):
    SUPPORTED_MODELS = {
        0x000001: ('828',       MotuProtocolV1),
        0x000002: ('828',       MotuProtocolV1),
        0x000003: ('828mk2',    MotuProtocolV2),
        0x000009: ('Traveler',  MotuProtocolV2),
        0x000015: ('828mk3',    MotuProtocolV3),    # FireWire only
        0x000035: ('828mk3',    MotuProtocolV3),    # Hybrid
        0x000033: ('AudioExpress', MotuProtocolV3),
    }

    def __init__(self, path):
        super().__init__()
        self.open(path)
        if self.get_property('type') != 7:
            raise ValueError('The character device is not for Motu unit.')

        ctx = GLib.MainContext.new()
        self.create_source().attach(ctx)
        self.__unit_dispatcher = GLib.MainLoop.new(ctx, False)
        self.__unit_th = Thread(target=lambda d: d.run(), args=(self.__unit_dispatcher, ))
        self.__unit_th.start()

        node = self.get_node()
        ctx = GLib.MainContext.new()
        node.create_source().attach(ctx)
        self.__node_dispatcher = GLib.MainLoop.new(ctx, False)
        self.__node_th = Thread(target=lambda d: d.run(), args=(self.__node_dispatcher, ))
        self.__node_th.start()

        parser = MotuConfigRomParser()
        info = parser.parse_rom(self.get_node().get_config_rom())

        if info['model-id'] in self.SUPPORTED_MODELS:
            name, protocol = self.SUPPORTED_MODELS[info['model-id']]
            self.name = name
            self._protocol = protocol(self, False)
        else:
            raise OSError('Unsupported model')

    def release(self):
        self.__unit_dispatcher.quit()
        self.__node_dispatcher.quit()
        self.__unit_th.join()
        self.__node_th.join()

    def __enter__(self):
        return self

    def __exit__(self, ex_type, ex_value, trace):
        self.release()

    def get_sampling_rates(self):
        return self._protocol.get_supported_sampling_rates()

    def get_sampling_rate(self):
        rate = self._protocol.get_sampling_rate()
        if rate not in self._protocol.get_supported_sampling_rates():
            raise RuntimeError('A bug of protocol abstraction, report this.')

        return rate

    def set_sampling_rate(self, rate):
        if self.get_property('streaming'):
            raise ValueError('Packet streaming already runs.')
        if rate not in self._protocol.get_supported_sampling_rates():
            raise ValueError('Unsupported sampling rate.')
        self._protocol.set_sampling_rate(rate)

    def get_supported_clock_sources(self):
        return self._protocol.get_supported_clock_sources()

    def get_clock_source(self):
        source = self._protocol.get_clock_source()
        if source not in self.get_supported_clock_sources():
            raise RuntimeError('A bug of protocol abstraction, report this.')

        return source

    def set_clock_source(self, source):
        if self.get_property('streaming'):
            raise ValueError('Packet streaming already runs.')
        if source not in self._protocol.get_supported_clock_sources():
            raise ValueError('Unsupported or unavailable clock source.')
        self._protocol.set_clock_source(source)

    #
    # Modes for optical interfaces.
    # This is for 828mk2 and Model dependent.
    #
    def get_supported_opt_iface_directions(self):
        return self._protocol.get_supported_opt_iface_directions()

    def get_opt_iface_modes(self):
        return self._protocol.get_supported_opt_iface_modes()

    def get_opt_iface_indexes(self):
        return self._protocol.get_supported_opt_iface_indexes()

    def get_opt_iface_mode(self, direction, index):
        if direction not in self._protocol.get_supported_opt_iface_directions():
            raise ValueError(
                'Invalid argument for direction of optical iface.')

        if index not in self._protocol.get_supported_opt_iface_indexes():
            raise ValueError('Invalid argument for index of optical iface.')

        mode = self._protocol.get_opt_iface_mode(direction, index)
        if mode not in self._protocol.get_supported_opt_iface_modes():
            raise RuntimeError('A bug of protocol abstraction, report this.')

        return mode

    def set_opt_iface_mode(self, direction, index, mode):
        if self.get_property('streaming'):
            raise ValueError('Packet streaming already runs.')

        if index not in self._protocol.get_supported_opt_iface_indexes():
            raise ValueError('Invalid argument for index of optical iface.')

        if mode not in self._protocol.get_supported_opt_iface_modes():
            raise ValueError('Invalid argument for mode of optical iface.')

        self._protocol.set_opt_iface_mode(direction, index, mode)



# SPDX-License-Identifier: LGPL-3.0-or-later
# Copyright (C) 2018 Takashi Sakamoto

import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="hinawa-utils",
    version="0.3.0",
    author="Takashi Sakamoto",
    author_email="o-takashi@sakamocchi.jp",
    description="Utility to control Audio and Music units on IEEE 1394 bus",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/takaswie/hinawa-utils",
    classifiers=(
        "Programming Language :: Python :: 3",
        "Operating System :: POSIX :: Linux",
        "License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
        "Development Status :: 4 - Beta",
        "Topic :: System :: Hardware :: Hardware Drivers",
        "Intended Audience :: Developers",
        "Environment :: Console",
    ),
    packages=(
        'hinawa_utils',
        'hinawa_utils.bebob',
        'hinawa_utils.dg00x',
        'hinawa_utils.dice',
        'hinawa_utils.efw',
        'hinawa_utils.fireface',
        'hinawa_utils.ieee1212',
        'hinawa_utils.ieee1394',
        'hinawa_utils.misc',
        'hinawa_utils.motu',
        'hinawa_utils.oxfw',
        'hinawa_utils.ta1394',
        'hinawa_utils.tscm',
    ),
    scripts=(
        'hinawa-alesis-io-cli',
        'hinawa-apogee-ensemble-cli',
        'hinawa-apogee-duet-cli',
        'hinawa-bebob-plug-parser',
        'hinawa-bebob-connection-cli',
        'hinawa-config-rom-printer',
        'hinawa-dg003-cli',
        'hinawa-dg00x-common-cli',
        'hinawa-dice-common-cli',
        'hinawa-dice-extension-cli',
        'hinawa-edirol-fa-cli',
        'hinawa-fireface-cli',
        'hinawa-fireworks-cli',
        'hinawa-focusrite-saffirepro-io-cli',
        'hinawa-griffin-firewave-cli',
        'hinawa-lacie-speakers-cli',
        'hinawa-maudio-bebob-cli',
        'hinawa-motu-common-cli',
        'hinawa-oxfw-generic-cli',
        'hinawa-tascam-fireone-cli',
        'hinawa-tascam-fw-console-cli',
        'hinawa-tascam-fw-rack-cli',
        'hinawa-yamaha-terratec-cli',
    ),
)



# SPDX-License-Identifier: LGPL-3.0-or-later
# Copyright (C) 2018 Takashi Sakamoto

from hinawa_utils.bebob.bebob_unit import BebobUnit
from hinawa_utils.ta1394.audio import AvcAudio

__all__ = ['EdirolFaUnit']


class EdirolFaUnit(BebobUnit):
    _FBS = {
        # Edirol FA-66.
        (0x0040ab, 0x010049): (
            'analog-in-1/2',
            'analog-in-3/4',
            'digital-in-1/2',
        ),
        # Edirol FA-101.
        (0x0040ab, 0x010048): (
            'analog-in-1/2',
            'analog-in-3/4',
            'analog-in-5/6',
            'analog-in-7/8',
            'digital-in-1/2',
        ),
    }

    def __init__(self, path):
        super().__init__(path)
        if (self.vendor_id, self.model_id) not in self._FBS:
            raise OSError('Not supported.')
        self._fbs = self._FBS[(self.vendor_id, self.model_id)]

    def get_mixer_input_labels(self):
        return self._fbs

    def set_mixer_input_gain(self, target, ch, gain):
        if target not in self._fbs:
            raise ValueError('Invalid argument for input.')
        fb = self._fbs.index(target) + 1
        data = AvcAudio.build_data_from_db(gain)
        AvcAudio.set_feature_volume_state(self.fcp, 0, 'current', fb, ch, data)

    def get_mixer_input_gain(self, target, ch):
        if target not in self._fbs:
            raise ValueError('Invalid argument for input.')
        fb = self._fbs.index(target) + 1
        data = AvcAudio.get_feature_volume_state(
            self.fcp, 0, 'current', fb, ch)
        return AvcAudio.parse_data_to_db(data)

    def set_mixer_input_balance(self, target, ch, balance):
        if target not in self._fbs:
            raise ValueError('Invalid argument for input.')
        fb = self._fbs.index(target) + 1
        data = AvcAudio.build_data_from_db(balance)
        AvcAudio.set_feature_lr_state(self.fcp, 0, 'current', fb, ch, data)

    def get_mixer_input_balance(self, target, ch):
        if target not in self._fbs:
            raise ValueError('Invalid argument for input.')
        fb = self._fbs.index(target) + 1
        data = AvcAudio.get_feature_lr_state(self.fcp, 0, 'current', fb, ch)
        return AvcAudio.parse_data_to_db(data)



# SPDX-License-Identifier: LGPL-3.0-or-later
# Copyright (C) 2018 Takashi Sakamoto

from abc import ABCMeta, abstractmethod

from hinawa_utils.ta1394.general import AvcConnection

__all__ = ['MaudioProtocolAbstract']


class MaudioProtocolAbstract(metaclass=ABCMeta):
    _METER_LABELS = ('analog-in-1', 'analog-in-2',
                     'digital-in-1', 'digital-in-2',
                     'analog-out-1', 'analog-out-2',
                     'analog-out-3', 'analog-out-4',
                     'analog-out-5', 'analog-out-6',
                     'analog-out-7', 'analog-out-8',
                     'digital-out-1', 'digital-out-2',
                     'headphone-out-1', 'headphone-out-2',
                     'headphone-out-3', 'headphone-out-4',
                     'switch-0', 'switch-1'
                     'rotery-0', 'rotery-1', 'rotery-2'
                     'rate', 'sync')
    _ADDR_FOR_METERING = 0xffc700600000

    def __init__(self, unit, debug):
        self.unit = unit

    # For input gains.
    @abstractmethod
    def get_input_labels(self):
        pass

    @abstractmethod
    def set_input_gain(self, target, ch, db):
        pass

    @abstractmethod
    def get_input_gain(self, target, ch):
        pass

    # For input LR balance.
    def get_input_balance_labels(self):
        pass

    def set_input_balance(self, target, ch, balance):
        pass

    def get_input_balance(self, target, ch):
        pass

    # For output volumes.
    @abstractmethod
    def get_output_labels(self):
        pass

    @abstractmethod
    def set_output_volume(self, target, ch, db):
        pass

    @abstractmethod
    def get_output_volume(self, target, ch):
        pass

    # For aux inputs.
    @abstractmethod
    def get_aux_input_labels(self):
        pass

    @abstractmethod
    def set_aux_input(self, target, ch, db):
        pass

    @abstractmethod
    def get_aux_input(self, target, ch):
        pass

    # For aux volume.
    @abstractmethod
    def set_aux_volume(self, ch, db):
        pass

    @abstractmethod
    def get_aux_volume(self, ch):
        pass

    # For headphone volume.
    @abstractmethod
    def get_headphone_labels(self):
        pass

    @abstractmethod
    def set_headphone_volume(self, target, ch, db):
        pass

    @abstractmethod
    def get_headphone_volume(self, target, ch):
        pass

    # For mixer routing.
    @abstractmethod
    def get_mixer_labels(self):
        pass

    @abstractmethod
    def get_mixer_source_labels(self):
        pass

    @abstractmethod
    def set_mixer_routing(self, target, source, enable):
        pass

    @abstractmethod
    def get_mixer_routing(self, target, source):
        pass

    # For headphone source.
    @abstractmethod
    def get_headphone_source_labels(self, target):
        pass

    @abstractmethod
    def set_headphone_source(self, target, source):
        pass

    @abstractmethod
    def get_headphone_source(self, target):
        pass

    # For output source.
    @abstractmethod
    def get_output_source_labels(self, target):
        pass

    @abstractmethod
    def set_output_source(self, target, source):
        pass

    @abstractmethod
    def get_output_source(self, target):
        pass

    # For metering.
    def get_meter_labels(self):
        return self._METER_LABELS

    @abstractmethod
    def get_meters(self):
        pass

    # For source of clock.
    @abstractmethod
    def get_clock_source_labels(self):
        pass

    @abstractmethod
    def set_clock_source(self, src):
        pass

    @abstractmethod
    def get_clock_source(self):
        pass

    # For sampling rate.
    def get_sampling_rate_labels(self):
        rates = []
        for rate in AvcConnection.SAMPLING_RATES:
            if not AvcConnection.ask_plug_signal_format(self.unit.fcp,
                                                        'input', 0, rate):
                continue
            if not AvcConnection.ask_plug_signal_format(self.unit.fcp,
                                                        'output', 0, rate):
                continue
            rates.append(rate)
        return rates

    def set_sampling_rate(self, rate):
        if self.unit.get_property('streaming'):
            raise ValueError('Packet streaming already runs.')
        if rate not in self.get_sampling_rate_labels():
            raise ValueError('Invalid argument for sampling rate')
        old_timeout = self.unit.fcp.get_property('timeout')
        # The unit tends to respond with larger interval from these requests.
        self.unit.fcp.set_property('timeout', 500)
        AvcConnection.set_plug_signal_format(self.unit.fcp, 'input', 0, rate)
        AvcConnection.set_plug_signal_format(self.unit.fcp, 'output', 0, rate)
        self.unit.fcp.set_property('timeout', old_timeout)

    @abstractmethod
    def get_sampling_rate(self):
        pass



# SPDX-License-Identifier: LGPL-3.0-or-later
# Copyright (C) 2018 Takashi Sakamoto

from abc import ABCMeta

from hinawa_utils.ta1394.general import AvcGeneral
from hinawa_utils.ta1394.general import AvcConnection
from hinawa_utils.ta1394.audio import AvcAudio

from hinawa_utils.bebob.extensions import BcoVendorDependent
from hinawa_utils.bebob.extensions import BcoPlugInfo
from hinawa_utils.bebob.extensions import BcoStreamFormatInfo

__all__ = ['PhaseGoProtocolAbstract']


class PhaseGoProtocolAbstract(metaclass=ABCMeta):
    __MIXER_INPUTS = {
        'analog-1/2':   6,
        'digital-1/2':  7,
        'stream-1/2':   3,
        'stream-3/4':   4,
        'stream-5/6':   5,
    }
    __OUTPUT_SOURCES = {
        'analog-1/2':   2,
        'digital-1/2':  3,
        'stream-1/2':   0,
        'stream-3/4':   1,
        'stream-5/6':   5,
        'mixer-1/2':    4,
    }
    __SUPPORTED_CLOCK_SOURCES = ('Internal', 'S/PDIF')
    __SUPPORTED_SAMPLING_RATES = (32000, 44100, 48000, 88200, 96000, 192000)

    def __init__(self, fcp, mixer_output_fb, output_labels):
        unit_info = AvcGeneral.get_unit_info(fcp)
        self.fcp = fcp
        self.__company_ids = unit_info['company-id']
        self.__mixer_output_fb = mixer_output_fb
        self.__output_labels = output_labels

    # For mixer inputs.
    def get_mixer_input_labels(self):
        return self.__MIXER_INPUTS

    def __check_mixer_input_channel(self, target, ch):
        if target not in self.__MIXER_INPUTS:
            raise ValueError('Invalid argument for stereo pair')
        if ch not in (1, 2):
            raise ValueError('Invalid argument for channel number')
        return self.__MIXER_INPUTS[target], ch

    def set_mixer_input_volume(self, target, ch, db):
        fb, ch = self.__check_mixer_input_channel(target, ch)
        data = AvcAudio.build_data_from_db(db)
        AvcAudio.set_feature_volume_state(self.fcp, 0, 'current', fb, ch, data)

    def get_mixer_input_volume(self, target, ch):
        fb, ch = self.__check_mixer_input_channel(target, ch)
        data = AvcAudio.get_feature_volume_state(
            self.fcp, 0, 'current', fb, ch)
        return AvcAudio.parse_data_to_db(data)

    def set_mixer_input_mute(self, target, ch, enable):
        fb, ch = self.__check_mixer_input_channel(target, ch)
        AvcAudio.set_feature_mute_state(self.fcp, 0, 'current', fb, ch, enable)

    def get_mixer_input_mute(self, target, ch):
        fb, ch = self.__check_mixer_input_channel(target, ch)
        return AvcAudio.get_feature_mute_state(self.fcp, 0, 'current', fb, ch)

    # For mixer output.
    def __check_mixer_output_channel(self, ch):
        if ch not in (1, 2):
            raise ValueError('Invalid argument for channel number')
        return self.__mixer_output_fb, ch

    def set_mixer_output_volume(self, ch, db):
        fb, ch = self.__check_mixer_output_channel(ch)
        data = AvcAudio.build_data_from_db(db)
        AvcAudio.set_feature_volume_state(self.fcp, 0, 'current', fb, ch, data)

    def get_mixer_output_volume(self, ch):
        fb, ch = self.__check_mixer_output_channel(ch)
        data = AvcAudio.get_feature_volume_state(
            self.fcp, 0, 'current', fb, ch)
        return AvcAudio.parse_data_to_db(data)

    def set_mixer_output_mute(self, ch, enable):
        fb, ch = self.__check_mixer_output_channel(ch)
        AvcAudio.set_feature_mute_state(self.fcp, 0, 'current', fb, ch, enable)

    def get_mixer_output_mute(self, ch):
        fb, ch = self.__check_mixer_output_channel(ch)
        return AvcAudio.get_feature_mute_state(self.fcp, 0, 'current', fb, ch)

    # For output sources.
    def get_output_labels(self):
        return list(self.__output_labels.keys())

    def get_output_source_labels(self):
        return list(self.__OUTPUT_SOURCES.keys())

    def set_output_source(self, target, source):
        if target not in self.__output_labels:
            raise ValueError('Invalid argumetn for output')
        if source not in self.__OUTPUT_SOURCES:
            raise ValueError('Invalid argumetn for output source')
        fb = self.__output_labels[target]
        ch = self.__OUTPUT_SOURCES[source]
        AvcAudio.set_selector_state(self.fcp, 0, 'current', fb, ch)

    def get_output_source(self, target):
        if target not in self.__output_labels:
            raise ValueError('Invalid argumetn for output')
        fb = self.__output_labels[target]
        idx = AvcAudio.get_selector_state(self.fcp, 0, 'current', fb)
        for key, val in self.__OUTPUT_SOURCES.items():
            if val == idx:
                return key
        else:
            raise ValueError('Unexpected value for output source.')

    def get_stream_formats(self):
        fmts = {}
        addr = BcoPlugInfo.get_unit_addr('output', 'isoc', 0)
        fmts['capture'] = BcoStreamFormatInfo.get_entry_list(self.fcp, addr)
        addr = BcoPlugInfo.get_unit_addr('input', 'isoc', 0)
        fmts['playback'] = BcoStreamFormatInfo.get_entry_list(self.fcp, addr)
        return fmts

    def check_digital_input_signal(self):
        # External input plug 1 is for S/PDIF in.
        return BcoVendorDependent.get_stream_detection(self.fcp,
                                                       self.__company_ids, 'input', 1)

    def get_clock_source_labels(self):
        return self.__SUPPORTED_CLOCK_SOURCES

    def set_clock_source(self, source):
        if source not in self.__SUPPORTED_CLOCK_SOURCES:
            raise ValueError('Invalid argument for clock source')
        val = self.__SUPPORTED_CLOCK_SOURCES.index(source)
        AvcAudio.set_selector_state(self.fcp, 0, 'current', 4, val)

    def get_clock_source(self):
        state = AvcAudio.get_selector_state(self.fcp, 0, 'current', 4)
        return self.__SUPPORTED_CLOCK_SOURCES[state]

    def get_sampling_rate_labels(self):
        return self.__SUPPORTED_SAMPLING_RATES

    def set_sampling_rate(self, rate):
        if rate not in self.__SUPPORTED_SAMPLING_RATES:
            raise ValueError('Invalid argument for sampling rate')
        AvcConnection.set_plug_signal_format(self.fcp, 'input', 0, rate)
        AvcConnection.set_plug_signal_format(self.fcp, 'output', 0, rate)

    def get_sampling_rate(self):
        return AvcConnection.get_plug_signal_format(self.fcp, 'output', 0)



# SPDX-License-Identifier: LGPL-3.0-or-later
# Copyright (C) 2018 Takashi Sakamoto

from hinawa_utils.tscm.tscm_unit import TscmUnit

__all__ = ['TscmConsoleUnit']


class TscmConsoleUnit(TscmUnit):
    def __init__(self, path):
        super().__init__(path)

        if self.model_name not in ('FW-1082', 'FW-1884'):
            raise ValueError('Unsupported model: {0}'.format(self.model_name))

    def bright_led(self, position, state):
        if state not in self.supported_led_status:
            raise ValueError('Invalid argument for LED state.')
        frames = bytearray(4)
        frames[3] = position
        if self.supported_led_status.index(state) == 1:
            frames[1] = 0x01
        self.write_quadlet(0x0404, frames)

    def set_master_fader(self, mode):
        frames = bytearray(4)
        if mode:
            frames[2] = 0x40
        else:
            frames[1] = 0x40
        self.write_quadlet(0x022c, frames)

    def get_master_fader(self):
        frames = self.read_quadlet(0x022c)
        return bool(frames[3] & 0x40)
